@@ -1,19 +1,8 @@
-# (C) British Crown Copyright 2010 - 2019, Met Office
+# Copyright Iris contributors
 #
-# This file is part of Iris.
-#
-# Iris is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Iris is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with Iris.  If not, see <http://www.gnu.org/licenses/>.
+# This file is part of Iris and is released under the LGPL license.
+# See COPYING and COPYING.LESSER in the root of the repository for full
+# licensing details.
 """
 Calculus operations on :class:`iris.cube.Cube` instances.
 
@@ -21,27 +10,25 @@ See also: :mod:`NumPy <numpy>`.
 
 """
 
-from __future__ import (absolute_import, division, print_function)
-from six.moves import (filter, input, map, range, zip)  # noqa
-import six
-
 import re
+import warnings
 
 import cf_units
 import numpy as np
 
-from iris._deprecation import warn_deprecated
 import iris.cube
 import iris.coords
 import iris.coord_systems
 import iris.analysis
 import iris.analysis.maths
-from iris.analysis.cartography import (DEFAULT_SPHERICAL_EARTH_RADIUS,
-                                       DEFAULT_SPHERICAL_EARTH_RADIUS_UNIT)
+from iris.analysis.cartography import (
+    DEFAULT_SPHERICAL_EARTH_RADIUS,
+    DEFAULT_SPHERICAL_EARTH_RADIUS_UNIT,
+)
 from iris.util import delta
 
 
-__all__ = ['cube_delta', 'differentiate', 'curl']
+__all__ = ["cube_delta", "differentiate", "curl"]
 
 
 def _construct_delta_coord(coord):
@@ -54,10 +41,12 @@ def _construct_delta_coord(coord):
     """
     if coord.ndim != 1:
         raise iris.exceptions.CoordinateMultiDimError(coord)
-    circular = getattr(coord, 'circular', False)
+    circular = getattr(coord, "circular", False)
     if coord.shape == (1,) and not circular:
-        raise ValueError('Cannot take interval differences of a single '
-                         'valued coordinate.')
+        raise ValueError(
+            "Cannot take interval differences of a single "
+            "valued coordinate."
+        )
 
     if circular:
         circular_kwd = coord.units.modulus or True
@@ -71,7 +60,7 @@ def _construct_delta_coord(coord):
 
     points = iris.util.delta(coord.points, 0, circular=circular_kwd)
     new_coord = iris.coords.AuxCoord.from_coord(coord).copy(points, bounds)
-    new_coord.rename('change_in_%s' % new_coord.name())
+    new_coord.rename("change_in_%s" % new_coord.name())
 
     return new_coord
 
@@ -84,21 +73,28 @@ def _construct_midpoint_coord(coord, circular=None):
     of length n-1.
 
     """
-    if circular and not hasattr(coord, 'circular'):
-        raise ValueError('Cannot produce circular midpoint from a coord '
-                         'without the circular attribute')
+    if circular and not hasattr(coord, "circular"):
+        msg = (
+            "Cannot produce a circular midpoint for the '{}' coord, "
+            "which does not have a 'circular' attribute."
+        )
+        raise ValueError(msg.format(coord.name()))
 
     if circular is None:
-        circular = getattr(coord, 'circular', False)
-    elif circular != getattr(coord, 'circular', False):
-        warn_deprecated('circular flag and Coord.circular attribute do '
-                        'not match')
+        circular = getattr(coord, "circular", False)
+    elif circular != getattr(coord, "circular", False):
+        msg = (
+            "Construction coordinate midpoints for the '{}' coordinate, "
+            "though it has the attribute 'circular'={}."
+        )
+        warnings.warn(msg.format(circular, coord.circular, coord.name()))
 
     if coord.ndim != 1:
         raise iris.exceptions.CoordinateMultiDimError(coord)
     if coord.shape == (1,) and not circular:
-        raise ValueError('Cannot take the midpoints of a single valued '
-                         'coordinate.')
+        raise ValueError(
+            "Cannot take the midpoints of a single valued " "coordinate."
+        )
 
     # Calculate the delta of the coordinate
     # (this deals with circularity nicely).
@@ -122,11 +118,13 @@ def _construct_midpoint_coord(coord, circular=None):
     # Try creating a coordinate of the same type as before, otherwise,
     # make an AuxCoord.
     try:
-        mid_point_coord = coord.from_coord(coord).copy(mid_point_points,
-                                                       mid_point_bounds)
+        mid_point_coord = coord.from_coord(coord).copy(
+            mid_point_points, mid_point_bounds
+        )
     except ValueError:
         mid_point_coord = iris.coords.AuxCoord.from_coord(coord).copy(
-            mid_point_points, mid_point_bounds)
+            mid_point_points, mid_point_bounds
+        )
 
     return mid_point_coord
 
@@ -153,7 +151,7 @@ cube_delta(temperature_cube, 'pressure')
 
     """
     # handle the case where a user passes a coordinate name
-    if isinstance(coord, six.string_types):
+    if isinstance(coord, str):
         coord = cube.coord(coord)
 
     if coord.ndim != 1:
@@ -161,19 +159,23 @@ cube_delta(temperature_cube, 'pressure')
 
     # Try and get a coord dim
     delta_dims = cube.coord_dims(coord.name())
-    if ((coord.shape[0] == 1 and not getattr(coord, 'circular', False)) or
-            not delta_dims):
-        raise ValueError('Cannot calculate delta over {!r} as it has '
-                         'length of 1.'.format(coord.name()))
+    if (
+        coord.shape[0] == 1 and not getattr(coord, "circular", False)
+    ) or not delta_dims:
+        raise ValueError(
+            "Cannot calculate delta over {!r} as it has "
+            "length of 1.".format(coord.name())
+        )
     delta_dim = delta_dims[0]
 
     # Calculate the actual delta, taking into account whether the given
     # coordinate is circular.
-    delta_cube_data = delta(cube.data, delta_dim,
-                            circular=getattr(coord, 'circular', False))
+    delta_cube_data = delta(
+        cube.data, delta_dim, circular=getattr(coord, "circular", False)
+    )
 
     # If the coord/dim is circular there is no change in cube shape
-    if getattr(coord, 'circular', False):
+    if getattr(coord, "circular", False):
         delta_cube = cube.copy(data=delta_cube_data)
     else:
         # Subset the cube to the appropriate new shape by knocking off
@@ -186,11 +188,15 @@ cube_delta(temperature_cube, 'pressure')
     # Replace the delta_dim coords with midpoints
     # (no shape change if circular).
     for cube_coord in cube.coords(dimensions=delta_dim):
-        delta_cube.replace_coord(_construct_midpoint_coord(
-            cube_coord, circular=getattr(coord, 'circular', False)))
+        delta_cube.replace_coord(
+            _construct_midpoint_coord(
+                cube_coord, circular=getattr(coord, "circular", False)
+            )
+        )
 
-    delta_cube.rename('change_in_{}_wrt_{}'.format(delta_cube.name(),
-                                                   coord.name()))
+    delta_cube.rename(
+        "change_in_{}_wrt_{}".format(delta_cube.name(), coord.name())
+    )
 
     return delta_cube
 
@@ -252,7 +258,7 @@ def differentiate(cube, coord_to_differentiate):
     # This operation results in a copy of the original cube.
     delta_cube = cube_delta(cube, coord_to_differentiate)
 
-    if isinstance(coord_to_differentiate, six.string_types):
+    if isinstance(coord_to_differentiate, str):
         coord = cube.coord(coord_to_differentiate)
     else:
         coord = coord_to_differentiate
@@ -264,8 +270,9 @@ def differentiate(cube, coord_to_differentiate):
     delta_cube = iris.analysis.maths.divide(delta_cube, delta_coord, delta_dim)
 
     # Update the standard name
-    delta_cube.rename('derivative_of_{}_wrt_{}'.format(cube.name(),
-                                                       coord.name()))
+    delta_cube.rename(
+        "derivative_of_{}_wrt_{}".format(cube.name(), coord.name())
+    )
     return delta_cube
 
 
@@ -429,16 +436,16 @@ def _trig_method(coord, trig_function):
 
     """
     # If we are in degrees create a copy that is in radians.
-    if coord.units == 'degrees':
+    if coord.units == "degrees":
         coord = coord.copy()
-        coord.convert_units('radians')
+        coord.convert_units("radians")
 
     trig_coord = iris.coords.AuxCoord.from_coord(coord)
     trig_coord.points = trig_function(coord.points)
     if coord.has_bounds():
         trig_coord.bounds = trig_function(coord.bounds)
-    trig_coord.units = '1'
-    trig_coord.rename('{}({})'.format(trig_function.__name__, coord.name()))
+    trig_coord.units = "1"
+    trig_coord.rename("{}({})".format(trig_function.__name__, coord.name()))
 
     return trig_coord
 
@@ -522,44 +529,47 @@ def curl(i_cube, j_cube, k_cube=None):
     """
     # Get the vector quantity names.
     # (i.e. ['easterly', 'northerly', 'vertical'])
-    vector_quantity_names, phenomenon_name = \
-        spatial_vectors_with_phenom_name(i_cube, j_cube, k_cube)
+    vector_quantity_names, phenomenon_name = spatial_vectors_with_phenom_name(
+        i_cube, j_cube, k_cube
+    )
 
     cubes = filter(None, [i_cube, j_cube, k_cube])
 
     # get the names of all coords binned into useful comparison groups
-    coord_comparison = iris.analysis.coord_comparison(*cubes)
+    coord_comparison = iris.analysis._dimensional_metadata_comparison(*cubes)
 
-    bad_coords = coord_comparison['ungroupable_and_dimensioned']
+    bad_coords = coord_comparison["ungroupable_and_dimensioned"]
     if bad_coords:
-        raise ValueError("Coordinates found in one cube that describe "
-                         "a data dimension which weren't in the other "
-                         "cube ({}), try removing this coordinate.".format(
-                             ', '.join(group.name() for group in bad_coords)))
+        raise ValueError(
+            "Coordinates found in one cube that describe "
+            "a data dimension which weren't in the other "
+            "cube ({}), try removing this coordinate.".format(
+                ", ".join(group.name() for group in bad_coords)
+            )
+        )
 
-    bad_coords = coord_comparison['resamplable']
+    bad_coords = coord_comparison["resamplable"]
     if bad_coords:
-        raise ValueError('Some coordinates are different ({}), consider '
-                         'resampling.'.format(
-                             ', '.join(group.name() for group in bad_coords)))
-
-    ignore_string = ''
-    if coord_comparison['ignorable']:
-        ignore_string = ' (ignoring {})'.format(
-            ', '.join(group.name() for group in bad_coords))
+        raise ValueError(
+            "Some coordinates are different ({}), consider "
+            "resampling.".format(
+                ", ".join(group.name() for group in bad_coords)
+            )
+        )
 
     # Get the dim_coord, or None if none exist, for the xyz dimensions
-    x_coord = i_cube.coord(axis='X')
-    y_coord = i_cube.coord(axis='Y')
-    z_coord = i_cube.coord(axis='Z')
+    x_coord = i_cube.coord(axis="X")
+    y_coord = i_cube.coord(axis="Y")
+    z_coord = i_cube.coord(axis="Z")
 
     y_dim = i_cube.coord_dims(y_coord)[0]
 
-    horiz_cs = i_cube.coord_system('CoordSystem')
+    horiz_cs = i_cube.coord_system("CoordSystem")
 
     # Non-spherical coords?
-    spherical_coords = isinstance(horiz_cs, (iris.coord_systems.GeogCS,
-                                  iris.coord_systems.RotatedGeogCS))
+    spherical_coords = isinstance(
+        horiz_cs, (iris.coord_systems.GeogCS, iris.coord_systems.RotatedGeogCS)
+    )
     if not spherical_coords:
 
         # TODO Implement some mechanism for conforming to a common grid
@@ -598,8 +608,8 @@ def curl(i_cube, j_cube, k_cube=None):
         di_dy = _curl_differentiate(i_cube, y_coord)
         di_dy = _curl_regrid(di_dy, prototype_diff)
         # Since prototype_diff == dj_dx we don't need to recalculate dj_dx
-#        dj_dx = _curl_differentiate(j_cube, x_coord)
-#        dj_dx = _curl_regrid(dj_dx, prototype_diff)
+        #        dj_dx = _curl_differentiate(j_cube, x_coord)
+        #        dj_dx = _curl_regrid(dj_dx, prototype_diff)
         k_cmpt = _curl_subtract(dj_dx, di_dy)
         di_dy = dj_dx = None
 
@@ -613,10 +623,18 @@ def curl(i_cube, j_cube, k_cube=None):
         #    (d/dtheta (i_cube * sin(lat)) - d_j_cube_dphi)
         # phi_cmpt = 1/r * ( d/dr (r * j_cube) - d_k_cube_dtheta)
         # theta_cmpt = 1/r * ( 1/cos(lat) * d_k_cube_dphi - d/dr (r * i_cube)
-        if y_coord.name() not in ['latitude', 'grid_latitude'] \
-                or x_coord.name() not in ['longitude', 'grid_longitude']:
-            raise ValueError('Expecting latitude as the y coord and '
-                             'longitude as the x coord for spherical curl.')
+        if (
+            y_coord.name()
+            not in [
+                "latitude",
+                "grid_latitude",
+            ]
+            or x_coord.name() not in ["longitude", "grid_longitude"]
+        ):
+            raise ValueError(
+                "Expecting latitude as the y coord and "
+                "longitude as the x coord for spherical curl."
+            )
 
         # Get the radius of the earth - and check for sphericity
         ellipsoid = horiz_cs
@@ -626,20 +644,21 @@ def curl(i_cube, j_cube, k_cube=None):
             # TODO: Add a test for this
             r = ellipsoid.semi_major_axis
             r_unit = cf_units.Unit("m")
-            spherical = (ellipsoid.inverse_flattening == 0.0)
+            spherical = ellipsoid.inverse_flattening == 0.0
         else:
             r = DEFAULT_SPHERICAL_EARTH_RADIUS
             r_unit = DEFAULT_SPHERICAL_EARTH_RADIUS_UNIT
             spherical = True
 
         if not spherical:
-            raise ValueError('Cannot take the curl over a non-spherical '
-                             'ellipsoid.')
+            raise ValueError(
+                "Cannot take the curl over a non-spherical " "ellipsoid."
+            )
 
         lon_coord = x_coord.copy()
         lat_coord = y_coord.copy()
-        lon_coord.convert_units('radians')
-        lat_coord.convert_units('radians')
+        lon_coord.convert_units("radians")
+        lat_coord.convert_units("radians")
         lat_cos_coord = _coord_cos(lat_coord)
 
         # TODO Implement some mechanism for conforming to a common grid
@@ -652,12 +671,14 @@ def curl(i_cube, j_cube, k_cube=None):
         # recalculate dicos_dtheta.
         d_j_cube_dphi = _curl_differentiate(j_cube, lon_coord)
         d_j_cube_dphi = _curl_regrid(d_j_cube_dphi, prototype_diff)
-        new_lat_coord = d_j_cube_dphi.coord(axis='Y')
+        new_lat_coord = d_j_cube_dphi.coord(axis="Y")
         new_lat_cos_coord = _coord_cos(new_lat_coord)
         lat_dim = d_j_cube_dphi.coord_dims(new_lat_coord)[0]
-        r_cmpt = iris.analysis.maths.divide(_curl_subtract(d_j_cube_dphi,
-                                                           dicos_dtheta),
-                                            r * new_lat_cos_coord, dim=lat_dim)
+        r_cmpt = iris.analysis.maths.divide(
+            _curl_subtract(d_j_cube_dphi, dicos_dtheta),
+            r * new_lat_cos_coord,
+            dim=lat_dim,
+        )
         r_cmpt.units = r_cmpt.units / r_unit
         d_j_cube_dphi = dicos_dtheta = None
 
@@ -671,7 +692,7 @@ def curl(i_cube, j_cube, k_cube=None):
         if drj_dr is None and d_k_cube_dtheta is None:
             phi_cmpt = None
         else:
-            phi_cmpt = 1/r * _curl_subtract(drj_dr, d_k_cube_dtheta)
+            phi_cmpt = 1 / r * _curl_subtract(drj_dr, d_k_cube_dtheta)
             phi_cmpt.units = phi_cmpt.units / r_unit
 
         drj_dr = d_k_cube_dtheta = None
@@ -680,8 +701,9 @@ def curl(i_cube, j_cube, k_cube=None):
         d_k_cube_dphi = _curl_differentiate(k_cube, lon_coord)
         d_k_cube_dphi = _curl_regrid(d_k_cube_dphi, prototype_diff)
         if d_k_cube_dphi is not None:
-            d_k_cube_dphi = iris.analysis.maths.divide(d_k_cube_dphi,
-                                                       lat_cos_coord)
+            d_k_cube_dphi = iris.analysis.maths.divide(
+                d_k_cube_dphi, lat_cos_coord
+            )
         dri_dr = _curl_differentiate(r * i_cube, z_coord)
         if dri_dr is not None:
             dri_dr.units = dri_dr.units * r_unit
@@ -689,7 +711,7 @@ def curl(i_cube, j_cube, k_cube=None):
         if d_k_cube_dphi is None and dri_dr is None:
             theta_cmpt = None
         else:
-            theta_cmpt = 1/r * _curl_subtract(d_k_cube_dphi, dri_dr)
+            theta_cmpt = 1 / r * _curl_subtract(d_k_cube_dphi, dri_dr)
             theta_cmpt.units = theta_cmpt.units / r_unit
         d_k_cube_dphi = dri_dr = None
 
@@ -697,7 +719,7 @@ def curl(i_cube, j_cube, k_cube=None):
 
     for direction, cube in zip(vector_quantity_names, result):
         if cube is not None:
-            cube.rename('%s curl of %s' % (direction, phenomenon_name))
+            cube.rename("%s curl of %s" % (direction, phenomenon_name))
 
     return result
 
@@ -718,33 +740,43 @@ def spatial_vectors_with_phenom_name(i_cube, j_cube, k_cube=None):
         (['u', 'v', 'w'], 'wind')
 
     """
-    directional_names = (('u', 'v', 'w'), ('x', 'y', 'z'), ('i', 'j', 'k'),
-                         ('eastward', 'northward', 'upward'),
-                         ('easterly', 'northerly', 'vertical'),
-                         ('easterly', 'northerly', 'radial'))
+    directional_names = (
+        ("u", "v", "w"),
+        ("x", "y", "z"),
+        ("i", "j", "k"),
+        ("eastward", "northward", "upward"),
+        ("easterly", "northerly", "vertical"),
+        ("easterly", "northerly", "radial"),
+    )
 
     # Create a list of the standard_names of our incoming cubes
     # (excluding the k_cube if it is None).
-    cube_standard_names = [cube.name() for cube in (i_cube, j_cube, k_cube)
-                           if cube is not None]
+    cube_standard_names = [
+        cube.name() for cube in (i_cube, j_cube, k_cube) if cube is not None
+    ]
 
     # Define a regular expr which represents (direction, phenomenon)
     # from the standard name of a cube.
     # e.g from "w wind" -> ("w", "wind")
-    vector_qty = re.compile(r'([^\W_]+)[\W_]+(.*)')
+    vector_qty = re.compile(r"([^\W_]+)[\W_]+(.*)")
 
     # Make a dictionary of {direction: phenomenon quantity}
     cube_directions, cube_phenomena = zip(
-        *[re.match(vector_qty, std_name).groups()
-            for std_name in cube_standard_names])
+        *[
+            re.match(vector_qty, std_name).groups()
+            for std_name in cube_standard_names
+        ]
+    )
 
     # Check that there is only one distinct phenomenon
     if len(set(cube_phenomena)) != 1:
-        raise ValueError('Vector phenomenon name not consistent between '
-                         'vector cubes. Got cube phenomena: {}; from '
-                         'standard names: {}.'.format(
-                             ', '.join(cube_phenomena),
-                             ', '.join(cube_standard_names)))
+        raise ValueError(
+            "Vector phenomenon name not consistent between "
+            "vector cubes. Got cube phenomena: {}; from "
+            "standard names: {}.".format(
+                ", ".join(cube_phenomena), ", ".join(cube_standard_names)
+            )
+        )
 
     # Get the appropriate direction list from the cube_directions we
     # have got from the standard name.
@@ -752,16 +784,20 @@ def spatial_vectors_with_phenom_name(i_cube, j_cube, k_cube=None):
     for possible_direction in directional_names:
         # If this possible direction (minus the k_cube if it is none)
         # matches direction from the given cubes use it.
-        if possible_direction[0:len(cube_directions)] == cube_directions:
+        if possible_direction[0 : len(cube_directions)] == cube_directions:
             direction = possible_direction
 
     # If we didn't get a match, raise an Exception
     if direction is None:
-        direction_string = '; '.join(', '.join(possible_direction)
-                                     for possible_direction
-                                     in directional_names)
-        raise ValueError('{} are not recognised vector cube_directions. '
-                         'Possible cube_directions are: {}.'.format(
-                             cube_directions, direction_string))
+        direction_string = "; ".join(
+            ", ".join(possible_direction)
+            for possible_direction in directional_names
+        )
+        raise ValueError(
+            "{} are not recognised vector cube_directions. "
+            "Possible cube_directions are: {}.".format(
+                cube_directions, direction_string
+            )
+        )
 
     return (direction, cube_phenomena[0])
